var Objects = require("convenience").Objects;
var _ = require("lodash");
var SettableFuture = com.google.common.util.concurrent.SettableFuture;
var Executors = java.util.concurrent.Executors;
var NotificationRequestHandler = Packages.cattura.web.api.vos.NotificationRequestHandler;
var facade = require("facade");
var cJSON = new Packages.cattura.agents.configuration.model.utilities.JSON();

/*
 * Stringify return value.
 */
function stringify (value)
{
    return cJSON.stringify(value).getOrRethrow();
}

/*
 * Parse function parameter names.
 */
function params (func)
{
    var funcStr = func.toString();
    var parsedParams = funcStr.substring(
        funcStr.indexOf("(") + 1,
        funcStr.indexOf(")")
    ).split(",");

    var parameterNames = [];
    for (var i = 0; i < parsedParams.length; i++)
    {
        parsedParams[i] = parsedParams[i].trim();

        if (parsedParams[i] !== "")
        {
            parameterNames.push(parsedParams[i]);
        }
    }

    return parameterNames;
}

/*
 * Create an API handler for a javascript callback.
 */
function collectArgs (funcParams, method, request, response, control)
{
    var params = Objects.fromMap(method.getParameters(request));
    var funcArgs = [];

    for (var i = 0; i < funcParams.length; i++)
    {
        var paramName = funcParams[i];
        var paramValue = null;
        if (paramName in params)
        {
            paramValue = Objects.toJS(params[paramName], true);
        }
        else if (paramName == "request" || _(paramName).endsWith("Request"))
        {
            paramValue = request;
        }
        else if (paramName == "response" || _(paramName).endsWith("Response"))
        {
            paramValue = response;
        }
        else if (paramName == "control" || _(paramName).endsWith("Control"))
        {
            paramValue = control;
        }

        funcArgs.push(paramValue);
    }
    
    return funcArgs;
}

function makeHandler (builder, apiProxy, method, callback)
{
    var HttpHandler = org.webbitserver.HttpHandler;
    var funcParams = params(callback);
    return new HttpHandler({
        handleHttpRequest: function (request, response, control) {
            try
            {
                var retval = callback.apply(apiProxy, collectArgs(funcParams, method, request, response, control));

                if (retval instanceof Response)
                {
                    retval.sendTo(request, response);
                }
                else
                {
                    retval = stringify(retval);
                    response.header("Content-Type","application/json").content(retval).end();
                }
            }
            catch (e)
            {
                if ("javaException" in e)
                {
                    facade.sendNotification("HANDLE_EXCEPTION", e.javaException);
                    builder.handleException(response, e.javaException);
                    return;
                }

                if (e instanceof APIError)
                {
                    response.status(e.code).header("Content-Type","application/json").content(stringify(e)).end();
                }
                else
                {
                    throw e;
                }
            }
        }
    });
}

/*
 * Define an API method.
 */
function method (builder, group, apiProxy, descriptor)
{
    var endpoint = descriptor.endpoint;
    var description = descriptor.description;
    var verb = descriptor.verb;
    var auth = descriptor.auth;
    var name = descriptor.name;
    var version = descriptor.version;

    var apiMethod = version ? group.method(name, endpoint, description, version)
                            : group.method(name, descriptor.endpoint, description);

    apiMethod.verb(verb);

    if (auth)
    {
        apiMethod.requireAuth(auth);
    }

    for (var paramName in descriptor.params)
    {
        /* jshint loopfunc:true */
        (function(key, param) {
            var type = "type" in param ? param.type : "any";
            var description = "description" in param ? param.description : key;
            if ('default' in param)
            {
                apiMethod.param(type, key, description, param['default']);
            }
            else
            {
                apiMethod.param(type, key, description);
            }
        })(paramName, descriptor.params[paramName]);
        /* jshint loopfunc:false */
    }

    if (descriptor.handler)
    {
        apiMethod.handler(descriptor.handler);
    }
    else if (descriptor.callback)
    {
        apiMethod.handler(makeHandler(builder, apiProxy, apiMethod, descriptor.callback));
    }
    else if (descriptor.notification)
    {
        apiMethod.handler(new NotificationRequestHandler(facade, descriptor.notification, apiMethod, cJSON));
    }

    return apiMethod;
}

/*
 * Return a wrapper for an API method that can be used to invoke as a javascript function.
 */
function wrapMethodAsFunction (apiProxy, descriptor)
{
    var callback = typeof descriptor === "function" ? descriptor : descriptor.callback;
    return function() { return callback.apply(apiProxy, arguments); };
}

/*
 * An API response that can be returned from a callback.
 */
function Response (params)
{
    if (typeof params === "function")
    {
        this.sendTo = params;
    }
    else
    {
        this.sendTo = function (httpRequest, httpResponse)
        {
            var contentType = 'contentType' in params ? params.contentType : "text/plain";
            var headers = 'headers' in params ? params.headers : {};
            var status = 'status' in params ? params.status : 200;
            var content = params.content;

            httpResponse.status(status).header("Content-Type", contentType);

            for (var key in headers)
            {
                if (headers.hasOwnProperty(key))
                {
                    httpResponse.header(key, headers[key]);
                }
            }

            if (typeof content === "function")
            {
                content = content(httpRequest);
            }

            httpResponse.content(content).end();
        };
    }
}

/*
 * An API error thrown from a callback.
 */
function APIError (code, message)
{
    this.code = code;
    this.message = message;
}

function PluginAPI (plugin)
{
    var apiBuilder = SettableFuture.create();
    var threadPool = Executors.newCachedThreadPool();

    if ('onEvent' in plugin)
    {
        plugin.onEvent(function(event) {
            if(event.getClass().toString().endsWith("BuildAPIEvent"))
            {
                apiBuilder.set(event.builder);
            }
        });
    }
    else if ('getClass' in plugin && plugin.getClass().toString().endsWith("APIBuilder"))
    {
        apiBuilder.set(plugin);
    }

    function buildParametersForAPIBuilder (apiDescription)
    {
        var parameters = {};
        for (var groupKey in apiDescription)
        {
            var groupDescriptor = apiDescription[groupKey];
            var title = 'title' in groupDescriptor ? groupDescriptor.title : groupKey;
            var description = 'description' in groupDescriptor ? groupDescriptor.description : "";
            var isPublic = 'isPublic' in groupDescriptor ? groupDescriptor.isPublic : true;
            var methods = 'methods' in groupDescriptor ? groupDescriptor.methods : {};
            var version = groupDescriptor.version;

            parameters[groupKey] = {
                title: title,
                description: description,
                version: version,
                isPublic: isPublic,
                methods: {}
            };

            for (var methodKey in methods)
            {
                parameters[groupKey].methods[methodKey] = buildMethodParametersForAPIBuilder(groupKey, methodKey, methods[methodKey]);
            }
        }
        return parameters;
    }

    function buildMethodParametersForAPIBuilder (groupKey, methodKey, descriptor)
    {
        var endpoint, description, verb, auth, name;
        if (typeof descriptor === "function")
        {
            descriptor = {
                callback: descriptor,
                verb: "post"
            };
            verb = "post";
            name = methodKey;
            endpoint = groupKey + "/" + methodKey;
        }

        endpoint = "endpoint" in descriptor ? descriptor.endpoint : groupKey + "/" + methodKey;
        description = "description" in descriptor ? descriptor.description : methodKey;
        verb = "verb" in descriptor ? descriptor.verb : "post";
        auth = "auth" in descriptor ? descriptor.auth : ("require" in descriptor ? descriptor.require : undefined);
        name = "name" in descriptor ? descriptor.name : methodKey;

        return {
            endpoint: endpoint,
            verb: verb,
            auth: auth,
            name: name,
            params: descriptor.params || {},
            version: descriptor.version,
            description: description,
            callback: descriptor.callback,
            handler: descriptor.handler,
            notification: descriptor.notification
        };
    }

    function buildAPIProxy (apiDescription)
    {
        var apiProxy = {};
        for (var groupKey in apiDescription)
        {
            apiProxy[groupKey] = {};
            var groupDescriptor = apiDescription[groupKey];
            var methods = 'methods' in groupDescriptor ? groupDescriptor.methods : {};

            for (var methodKey in methods)
            {
                var descriptor = methods[methodKey];
                var functionWrapper = wrapMethodAsFunction(apiProxy, descriptor);
                apiProxy[groupKey][methodKey] = functionWrapper;

                if (descriptor.name)
                {
                    apiProxy[groupKey][descriptor.name] = functionWrapper;
                }
            }
        }
        return apiProxy;
    }

    function buildAPI (builder, parameters, proxy)
    {
        for (var groupKey in parameters)
        {
            var groupParameters = parameters[groupKey];
            var group = builder.group(
                groupKey,
                groupParameters.title,
                groupParameters.description,
                groupParameters.version,
                groupParameters.isPublic
            );

            for (var methodKey in groupParameters.methods)
            {
                method(builder, group, proxy, groupParameters.methods[methodKey]);
            }
        }
    }

    var API = function API (api)
    {
        var apiProxy = buildAPIProxy(api);
        var apiBuilderParameters = buildParametersForAPIBuilder(api);
        apiBuilder.addListener(function() {
            buildAPI(apiBuilder.get(), apiBuilderParameters, apiProxy);
        }, threadPool);

        return apiProxy;
    };

    API.Response = Response;
    API.Error = APIError;
    return API;
}

PluginAPI.Response = Response;
PluginAPI.Error = APIError;

module.exports = PluginAPI;
