/* jshint undef: true, unused: true, esnext: true */
/* global module, require, Packages */
/* global callPluginServiceHandler */

var facade = require("facade");
var Objects = require("convenience").Objects;
var _ = require("lodash");

var API = require("./apiHelper");

var Permissions = Packages.cattura.web.server.properties.Permissions;
var PluginProxy = Packages.cattura.agents.configuration.model.proxies.PluginProxy;

function serializeNotifier (plugin, notifier)
{
    if (!notifier)
    {
        return null;
    }
 
    if (typeof plugin === "object")
    {
        plugin = plugin.pluginID;
    }
    
    return _({ pluginId: plugin })
        .extend(notifier)
        .omitBy(_.isFunction).value();
}

function getNotifiers ()
{
    var pluginProxy = facade.retrieveProxy(PluginProxy.NAME);
    return _(Objects.toJS(pluginProxy.getPluginInfos())).filter(function (plugin) {
        return plugin.type && plugin.type.toLowerCase() == "notifier" && !plugin.getServicesByName("getNotifiers").isEmpty();
    }).map(function (plugin) {
        return _(Objects.toJS(callPluginServiceHandler.invokeService(plugin.pluginID, "getNotifiers", "{}"), true))
            .map(notifier => serializeNotifier(plugin, notifier))
            .value();
    }).flatten().value();
}

function getNotifierPlugin (pluginId)
{
    var pluginProxy = facade.retrieveProxy(PluginProxy.NAME);
    return _(Objects.toJS(pluginProxy.getPluginInfos())).find(function (plugin) {
        return plugin.pluginID == pluginId && plugin.type && plugin.type.toLowerCase() == "notifier" && !plugin.getServicesByName("addNotifier").isEmpty();
    });
}

function getNotifier (id)
{
    var notifier = _(getNotifiers()).find(function (notifier) {
        return notifier.id == id;
    });

    if (!notifier)
    {
        throw new API.Error(404);
    }

    return notifier;
}

var api = {
    title: "Notifiers",
    description: "Methods for managing registered notifiers.",
    version: 1,
    methods: {
        getNotifiers: {
            name: "getNotifiers",
            description: "Get all notifiers registered in CaptureCast.",
            verb: "get",
            require: Permissions.MANAGE_SETTINGS,
            endpoint: "notifiers",
            callback: getNotifiers
        },
        getNotifier: {
            name: "getNotifier",
            description: "Get a single notifier registered in CaptureCast.",
            verb: "get",
            require: Permissions.MANAGE_SETTINGS,
            endpoint: "notifiers/{id}",
            params: {
                id: {type: "string", description: "the ID of the notifier to get"}
            },
            callback: getNotifier
        },
        deleteNotifier: {
            name: "deleteNotifier",
            description: "Delete a notifier from CaptureCast.",
            verb: "delete",
            require: Permissions.MANAGE_SETTINGS,
            endpoint: "notifiers/{id}",
            params: {
                id: {type: "string", description: "the ID of the notifier to delete"},
            },
            callback: function (id) {
                var notifier = getNotifier(id);
                callPluginServiceHandler.invokeService(notifier.pluginId, "removeNotifier", JSON.stringify({
                    id: id
                }));
                return notifier;
            }
        },
        createNotifier: {
            name: "createNotifier",
            description: "Create a notifier in CaptureCast.",
            verb: "post",
            require: Permissions.MANAGE_SETTINGS,
            endpoint: "notifiers",
            params: {
                plugin: {type: "string", description: "the ID of the plugin to create the notifier with"},
                args: {type: "json", description: "arguments for the notifier, as a JSON string", "default": "{}"}
            },
            callback: function (plugin, args) {
                var pluginId = plugin;
                plugin = getNotifierPlugin(pluginId);
                
                if (!plugin)
                {
                    throw new API.Error(400, "No such notifier plugin: " + pluginId);
                }
                
                return serializeNotifier(plugin, callPluginServiceHandler.invokeService(plugin.pluginID, "addNotifier", args));
            }
        },
        updateNotifier: {
            name: "updateNotifier",
            description: "Update an existing notifier in CaptureCast.",
            verb: "post",
            require: Permissions.MANAGE_SETTINGS,
            endpoint: "notifiers/{id}",
            params: {
                id: {type: "string", description: "the ID of the notifier to update"},
                args: {type: "json", description: "arguments for the notifier, as a JSON string", "default": "{}"}
            },
            callback: function (id, plugin, args) {
                var notifier = getNotifier(id);
                args = JSON.parse(args);
                args.id = id;
                
                return serializeNotifier(notifier.pluginId + "", callPluginServiceHandler.invokeService(notifier.pluginId, "addNotifier", JSON.stringify(args)));
            }
        }
    }
};

module.exports = api;
