/* jshint undef: true, unused: true */
/* global module, require, Packages */
/* global callPluginServiceHandler */

var facade = require("facade");
var Objects = require("convenience").Objects;
var _ = require("lodash");

var SETTINGS = facade.retrieveProxy("SettingsProxy");

var Permissions = Packages.cattura.web.server.properties.Permissions;
var PluginProxy = Packages.cattura.agents.configuration.model.proxies.PluginProxy;

var api = {
    title: "Plugins",
    description: "Provides information about, and methods to manage, plugins installed in CaptureCast.",
    version: 1,
    methods: {
        get: {
            name: "getPlugins",
            description: "Returns a list of all plugins installed in CaptureCast.",
            verb: "get",
            endpoint: "plugins",
            callback: function() {
                var mediaProcessingModule = facade.retrieveProxy("MediaProcessingProxy").getCaptureModule();
                var pluginProxy = facade.retrieveProxy(PluginProxy.NAME);
                var plugins = {};

                /*
                 * MPA module plugins
                 */
                if (mediaProcessingModule && mediaProcessingModule.hasFoundPlugins())
                {
                    _(Objects.toJS(mediaProcessingModule.getPlugins())).each(function(plugin) {
                        var services = {};

                        _(Objects.toJS(plugin.services)).each(function(service) {
                            var parameters = {};

                            _(Objects.toJS(service.parameters)).map(function(parameter) {
                                parameters[parameter.name] = {
                                    name: parameter.name,
                                    description: parameter.description,
                                    type: parameter.type
                                };
                            }).value();

                            services[service.name] = {
                                name: service.name,
                                description: service.description,
                                parameters: parameters
                            };
                        });

                        plugins[plugin.ID] = {
                            id: plugin.ID,
                            type: plugin.type.toString(),
                            title: plugin.title,
                            module: "media-processing-agent",
                            description: plugin.description,
                            copyright: plugin.copyright,
                            options: JSON.parse(plugin.optionsAsString.get().content),
                            services: services,
                            inputInterests: plugin.inputInterests,
                            outputInterests: plugin.outputInterests,
                            alsoKnownAs: plugin.alsoKnownAs
                        };
                    });
                }

                /*
                 * Configuration Module plugins
                 */
                _(Objects.toJS(pluginProxy.getPluginInfos())).each(function(plugin) {
                    var services = {};

                    _(Objects.toJS(plugin.services)).each(function(service) {
                        var parameters = {};

                        _(Objects.toJS(service.parameters)).map(function(parameter) {
                            parameters[parameter.name] = {
                                name: parameter.name,
                                description: parameter.description,
                                type: parameter.parameterType.name
                            };
                        }).value();

                        services[service.name] = {
                            name: service.name,
                            description: service.description,
                            parameters: parameters
                        };
                    });

                    plugins[plugin.pluginID] = {
                        id: plugin.pluginID,
                        type: plugin.type || "configuration", // might need to be able to specify this
                        title: plugin.title,
                        module: "configuration",
                        description: plugin.description,
                        copyright: plugin.copyright,
                        services: services,
                        alsoKnownAs: [],
                        options: {} // Eventually we'll define these for CM plugins
                    };
                });

                return plugins;
            }
        },

        getSettingsForPlugin: {
            name: "getSettingsForPlugin",
            description: "Retrieve user settings for a plugin.",
            verb: "get",
            require: Permissions.VIEW,
            endpoint: "plugins/{plugin}/settings",
            params: {
                plugin: {type: "string", description: "the ID of the plugin"}
            },
            callback: function(plugin) {
                var pluginSettings = SETTINGS.getUserSettingsForPluginId(plugin).orElse(null);

                if (pluginSettings)
                {
                    return pluginSettings.toMap();
                }

                return {};
            }
        },

        setSettingsForPlugin: {
            name: "setSettingsForPlugin",
            description: "Retrieve user settings for a plugin.",
            verb: "post",
            require: Permissions.MANAGE_SETTINGS,
            endpoint: "plugins/{plugin}/settings",
            params: {
                plugin: {type: "string", description: "the ID of the plugin to invoke the service on"},
                settings: {type: "json", description: "key/value mapping of settings to set on the plugin"}
            },
            callback: function(plugin, settings) {
                var pluginSettings = SETTINGS.getUserSettingsForPluginId(plugin).orElse(null);
                settings = JSON.parse(settings);

                if (pluginSettings)
                {
                    _(settings).each(function (value, key) {
                        pluginSettings.set(key, value);
                    });
                    return pluginSettings.toMap();
                }

                return {};
            }
        },

        callService: {
            name: "callPluginService",
            description: "Invoke a service provided by a plugin",
            verb: "post",
            require: Permissions.VIEW,
            endpoint: "plugins/{plugin}/{service}",
            params: {
                plugin: {type: "string", description: "the ID of the plugin to invoke the service on"},
                service: {type: "string", description: "the name of the service to call"},
                args: {type: "json", description: "arguments for the service, as a JSON string", "default": "{}"}
            },
            handler: callPluginServiceHandler
        }
    }
};

module.exports = api;
