/* jshint undef: true, unused: true */
/* global module, require, Packages */

var facade = require("facade");
var Objects = require("convenience").Objects;

var OSSystemState = Packages.cattura.utils.os.OSSystemState;
var SystemInformationUtilities = Packages.cattura.agents.configuration.model.utilities.SystemInformationUtilities;
var Permissions = Packages.cattura.web.server.properties.Permissions;

var SETTINGS = facade.retrieveProxy("SettingsProxy");
var systemStatusProxy = facade.retrieveProxy("EnvironmentProxy");
var captureStateProxy = facade.retrieveProxy("CaptureStateProxy");
var deviceProxy = facade.retrieveProxy("DeviceProxy");

var ExceptionUtils = Packages.cattura.utils.exception.ExceptionUtils;

function getDiskSpaceInfo ()
{
    var diskSpaceWatcher = systemStatusProxy.diskSpaceWatcher;
    return {
        total: diskSpaceWatcher.totalSpace,
        usable: diskSpaceWatcher.usableSpace,
        free: diskSpaceWatcher.freeSpace,
        used: diskSpaceWatcher.usedSpace,
        low: diskSpaceWatcher.isDiskSpaceLow()
    };
}

function getNetworkInfo ()
{
    var networkWatcher = systemStatusProxy.networkStatusWatcher;
    return {
       internet: {
           connected: networkWatcher.hasInternet(),
           ip: networkWatcher.globalIP
       },
       local: {
           ip: networkWatcher.localIP,
           type: networkWatcher.networkType,
           subnet: networkWatcher.subnet,
           gateway: networkWatcher.gateway,
           mac: networkWatcher.getMACAddress()
       }
    };
}

function getConnectedDevices ()
{
    var deviceStructure = {};
    var audioDevices = Objects.toJS(deviceProxy.audioDevices);
    var videoDevices = Objects.toJS(deviceProxy.videoDevices);
    var key;

    for (key in audioDevices)
    {
        deviceStructure[key] = {
            id: audioDevices[key].name,
            name: audioDevices[key].prettyName,
            type: "audio"
        };
    }

    for (key in videoDevices)
    {
        deviceStructure[key] = {
            id: videoDevices[key].name,
            name: videoDevices[key].prettyName,
            type: "video",
            snapshot: "/thumbnail/" + key,
            liveStream: "/live/" + key,
            width: videoDevices[key].width,
            height: videoDevices[key].height,
            fps: videoDevices[key].FPS
        };
    }

    return deviceStructure;
}

function getCaptureState ()
{
    var captureState = captureStateProxy.captureState;
    var stateStructure = {
        state: captureState.state.name()
    };

    if (stateStructure.state != "IDLE")
    {
        stateStructure.currentCapture = {
            title: captureState.captureRequest.title,
            startTime: captureState.captureRequest.creationTime.time,
            metadata: captureState.captureRequest.metadata,
            tags: captureState.captureRequest.tags
        };

        if (captureState.session)
        {
            stateStructure.currentCapture.paused = captureState.session.paused;
            stateStructure.currentCapture.elapsed = captureState.session.elapsedTimeMillis;
        }
    }

    return stateStructure;
}

function getUnitInfo ()
{
    var updaterProxy = facade.retrieveProxy("UpdaterProxy");
    var versionInfo = updaterProxy.getVersionInfo();

    return {
        unitName: SETTINGS.getUserSettings().getString("unit-name").orElse(null),
        machineName: OSSystemState.getMachineName(),
        uuid: SystemInformationUtilities.getSystemUUID(),
        agentVersion: versionInfo.getCurrentVersion(),
        serial: SETTINGS.getSystemSettings().get("serial").orElse(null)
    };
}

function getTodaysSchedule ()
{
    var schedulerProxy = facade.retrieveProxy("SchedulerProxy");
    var scheduleStructure = [];
    var events = Objects.toJS(schedulerProxy.todaysEvents);

    for (var i = 0; i < events.length; i++)
    {
        var event = events[i];
        var task = schedulerProxy.getScheduler().getTaskSource().getTaskForEvent(event);
        scheduleStructure.push({
            id: event.ID,
            group: event.groupID,
            title: event.title,
            start: event.startTime.time,
            stop: event.stopTime.time,
            templateID: task.templateID,
            mediaPackageID: task.mediaPackageID
        });
    }

    return scheduleStructure;
}

function getStartupErrors ()
{
    var applicationStatusProxy = facade.retrieveProxy("ApplicationStatusProxy");
    var exceptionStructure = [];
    var exceptions = Objects.toJS(applicationStatusProxy.getStartupExceptions());
    
    for (var i = 0; i < exceptions.length; i++)
    {
        exceptionStructure.push(Objects.toJS(ExceptionUtils.convertToMap(exceptions[i]), true));
    }
    
    return exceptionStructure;
}

var api = {
    title: "Unit Status",
    description: "Provides methods to query the current state of the recorder.",
    version: 1,
    methods: {
        general: {
            name: "getUnitStatus",
            description: "Obtain general information about the current state of the recorder.",
            verb: "get",
            //require: "Administrator",
            endpoint: "status",
            params: {
                since: {type: "long", description: "No longer used", "default": -1},
            },
            callback: function () {
                return {
                    storage: getDiskSpaceInfo(),
                    network: getNetworkInfo(),
                    devices: getConnectedDevices(),
                    capture: getCaptureState(),
                    info: getUnitInfo(),
                    schedule: getTodaysSchedule(),
                };
            }
        },

        unit: {
            name: "getUnitInfo",
            description: "Get basic information about the recorder.",
            verb: "get",
            require: Permissions.VIEW,
            endpoint: "status/unit",
            callback: getUnitInfo
        },

        network: {
            name: "getNetworkInfo",
            description: "Gets information about the network connectivity of the recorder.",
            verb: "get",
            require: Permissions.VIEW,
            endpoint: "status/network",
            callback: getNetworkInfo
        },

        space: {
            name: "getDiskSpaceInfo",
            description: "Gets the amount of free, usable, and total disk space.",
            verb: "get",
            require: Permissions.VIEW,
            endpoint: "status/space",
            callback: getDiskSpaceInfo
        },

        captureState: {
            name: "getCaptureState",
            description: "Returns the current capture's state (idle, capturing, processing, etc)",
            verb: "get",
            require: Permissions.VIEW,
            endpoint: "status/capture",
            callback: getCaptureState
        },

        connectedDevices: {
            name: "getConnectedDevices",
            description: "Returns a listing of audio and video devices connected to this recorder.",
            verb: "get",
            require: Permissions.VIEW,
            endpoint: "status/devices",
            callback: getConnectedDevices
        },

        todaysSchedule: {
            name: "getTodaysSchedule",
            description: "Returns all the scheduled captures for today.",
            verb: "get",
            require: Permissions.VIEW,
            endpoint: "status/schedule",
            callback: getTodaysSchedule
        },
        
        startupErrors: {
            name: "getStartupErrors",
            description: "Returns all errors that occurred when CaptureCast started up.",
            verb: "get",
            endpoint: "status/startupErrors",
            callback: getStartupErrors
        }
    }
};

module.exports = api;
