/* jshint esnext: true */

const _ = require("lodash");
const ko = require("knockout");

const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const selectable = require("cattura-ui-library/src/lib/ui/selectable");

const template = require("./content.pug");

const {apiv1} = require("./api");

const DEFAULT_ROLE_DESCRIPTIONS = {
    PRESENTER: "Can start/stop capture and view media",
    STATUS_VIEWER: "Limited view-only access to dashboard and media section.",
    CONTENT_MANAGER: "Can start/stop capture and has full access to manage media section.",
    SCHEDULER: "Can start/stop capture and manage the schedule.",
    ADMINISTRATOR: "Can start/stop capture, manage templates, media, and configure CaptureCast."
};

const PECKING_ORDER = [
    "STATUS_VIEWER", "PRESENTER", "SCHEDULER", "CONTENT_MANAGER", "ADMINISTRATOR"
];

let roles = selectable.selectableObservableArray();
function fetchRoles ()
{
    return apiv1.users.getRoles().then(function (retval) {
        roles(_(retval).toPairs().sortBy(function (pair) {
            return PECKING_ORDER.indexOf(pair[0]);
        }).map(function (pair) {
            return _(pair[1]).extend({
                id: pair[0],
                description: DEFAULT_ROLE_DESCRIPTIONS[pair[0]] || pair[1].permissions.join(",")
            }).value();
        }).map(selectable.makeSelectable).value());
        return roles();
    });
}

let keyId = ko.observable();
let label = ko.observable().extend({
    required: true
});

const addEditUser = Dialog({
    id: keyId,

    fields: ko.validatedObservable({
        label: label
    }),

    roles: roles,

    open (key)
    {
        let roles = this.roles().length === 0 ? fetchRoles()
                                              : Promise.resolve(this.roles());

        this.reset();

        if ("id" in key)
        {
            this.id(key.id);
            this.fields().label(key.label);
            
            roles.then(roles => {
                _(roles).filter(role => _(key.roles).find(userRole => userRole.name === role.name))
                        .each(role => role.selected(true));
            });
        }

        this.show();
    },

    reset ()
    {
        this.id(null);
        this.fields().label("");
        this.roles.deselectAll();
    },

    save ()
    {
        if (!this.fields.isValid())
        {
            this.dispatchEvent({type: "error", message: "Form is not valid."});
            return;
        }
        else if (this.roles.selected().length === 0)
        {
            this.dispatchEvent({type: "error", message: "At least one role must be selected."});
            return;
        }
        
        let request;
        if (this.id())
        {
            request = apiv1.api.updateKey({
                id: this.id(),
                label: this.fields().label(),
                roles: _(this.roles.selected()).map("id").join(","),
                permissions: ""
            });
        }
        else
        {
            request = apiv1.api.addKey({
                label: this.fields().label(),
                roles: _(this.roles.selected()).map("id").join(","),
                permissions: ""
            });
        }

        request.then(retval => {
            this.dispatchEvent({type: "keysaved", key: retval});
            this.hide();
        }).catch(error => this.dispatchEvent({type: "error", message: error.message, error: error}));
    }
}, {
    template: template
});

module.exports = addEditUser;
